/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ObservationListAdapter.java,v 968db7f872cb 2010/06/30 01:21:58 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.observation;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import android.content.Context;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;
import android.widget.TextView;
import it.tidalwave.util.AsException;
import it.tidalwave.util.Parameters;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.geo.Range;
import it.tidalwave.mobile.location.LocationPreferences;
import it.tidalwave.bluebill.observation.Location;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.ObservationSet;
import it.tidalwave.bluebill.observation.ObservationSet.Event;
import it.tidalwave.bluebill.mobile.GeneralPreferences;
import it.tidalwave.bluebill.mobile.android.R;
import it.tidalwave.bluebill.mobile.observation.ObservationPreferences;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonomyPreferences;
import static it.tidalwave.bluebill.observation.Observation.Observation;
import static it.tidalwave.bluebill.taxonomy.Taxon.Taxon;
import static it.tidalwave.bluebill.mobile.observation.ObservationPreferences.Extra.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class ObservationListAdapter extends BaseExpandableListAdapter
  {
    private static final String CLASS = ObservationListAdapter.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private static final Class<Range> Range = Range.class; // FIXME: move to Range

    @Nonnull
    private final GeneralPreferences generalPreferences = Locator.find(GeneralPreferences.class);

    private final List<Observation> observations = new ArrayList<Observation>();

    @Nonnull
    private final TaxonomyPreferences taxonomyPreferences = Locator.find(TaxonomyPreferences.class);
    
    @Nonnull
    private final Context context = Locator.find(Context.class);

    @Nonnull
    private final LayoutInflater layoutInflater;

    @Nonnull
    private final ObservationSet observationSet;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final ObservationSet.Listener listener = new ObservationSet.Listener()
      {
        public void notifyObservationAdded (final @Nonnull Event event)
          {
            recompute();
            notifyDataSetChanged();
          }

        public void notifyObservationRemoved (final @Nonnull Event event)
          {
            recompute();
            notifyDataSetChanged();
          }

        public void notifyObservationChanged (final @Nonnull Event event)
          {
            recompute();
            notifyDataSetChanged();
          }

        public void notifyCleared (final @Nonnull Event event)
          {
            recompute();
            notifyDataSetChanged();
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final Comparator<Observation> observationComparator = new Comparator<Observation>()
      {
        public int compare (final @Nonnull Observation observation1, final @Nonnull Observation observation2)
          {
            return observation1.getDate().compareTo(observation2.getDate());
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final Comparator<ObservationItem> observationItemComparator = new Comparator<ObservationItem>()
      {
        public int compare (final @Nonnull ObservationItem item1, final @Nonnull ObservationItem item2)
          {
            return taxonomyPreferences.getTaxonComparator().compare(item1.getObservable().as(Taxon),
                                                                    item2.getObservable().as(Taxon));
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public ObservationListAdapter (final @Nonnull ObservationSet observationSet)
      {
        Parameters.checkNonNull(context, "context");
        Parameters.checkNonNull(observationSet, "observationSet");

        this.observationSet = observationSet;
        this.layoutInflater = LayoutInflater.from(context);
        this.observationSet.addListener(listener);
        recompute();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Observation getGroup (final @Nonnegative int position)
      {
        return observations.get(position);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnegative
    public int getGroupCount()
      {
        return observations.size();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public long getGroupId (final @Nonnegative int position)
      {
        return position;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public ObservationItem getChild (final @Nonnegative int groupPosition,
                                     final @Nonnegative int childPosition)
      {
        final List<? extends ObservationItem> items = getGroup(groupPosition).findObservationItems().results();
        // FIXME: resorting every time is not the most efficient strategy. But probably only a handful of things
        // needs to be sorted in a while, and a smarter implementatiom might be useless. Check.
        final long time = System.currentTimeMillis();
        Collections.sort(items, observationItemComparator);
        logger.info("taxa sorted in %d msec", System.currentTimeMillis() - time);
        return items.get(childPosition);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnegative
    public int getChildrenCount (final @Nonnegative int groupPosition)
      {
        return getGroup(groupPosition).findObservationItems().count();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public long getChildId (final @Nonnegative int groupPosition,
                            final @Nonnegative int childPosition)
      {
        return childPosition;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public View getGroupView (final @Nonnegative int groupPosition,
                              final boolean isExpanded,
                              @CheckForNull View view,
                              final @Nonnull ViewGroup parent)
      {
        TextView textView;

        if (view == null)
          {
            view = layoutInflater.inflate(R.layout.observation_browser_row, null);
            textView = (TextView)view.findViewById(R.id.tvText);
            view.setTag(textView);
          }
        else
          {
            textView = (TextView)view.getTag();
          }

        final Observation observation = getGroup(groupPosition);
        final StringBuilder builder = new StringBuilder();
        builder.append(generalPreferences.formatDateAndTime(observation.getDate(), "<b>", "</b>"));

        final Location location = observation.getLocation();

        if (location != null) // shouldn't happen, but it does because of various bugs
          {
            builder.append("<br>");
            builder.append(location.getDisplayName());

            try
              {
                final Range range = location.as(Range);
                final LocationPreferences locationPreferences = Locator.find(LocationPreferences.class);
                builder.append(String.format("<br><small>%s</small>", locationPreferences.format(range)));
              }
            catch (AsException e)
              {
                // range not available, never mind
              }
          }

        textView.setText(Html.fromHtml(builder.toString()));
        
        return view;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public View getChildView (final @Nonnegative int groupPosition,
                              final @Nonnegative int childPosition,
                              final boolean isLastChild,
                              @CheckForNull View view,
                              final @Nonnull ViewGroup parent)
      {
        TextView textView;

        if (view == null)
          {
            view = layoutInflater.inflate(R.layout.observationitem_browser_row, null);
            textView = (TextView)view.findViewById(R.id.tvText);
            view.setTag(textView);
          }
        else
          {
            textView = (TextView)view.getTag();
          }

        final ObservationPreferences observablePreferences = Locator.find(ObservationPreferences.class);
        final ObservationItem observationItem = getChild(groupPosition, childPosition);
        textView.setText(observablePreferences.formatAsHtml(observationItem, CARDINALITY, GENDER, MODE, NOTE_FULL));

        return view;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isChildSelectable (final @Nonnegative int groupPosition,
                                      final @Nonnegative int childPosition)
      {
        return true;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean hasStableIds()
      {
        return true;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void recompute()
      {
        observations.clear();
        observations.addAll(observationSet.find(Observation).results());
        final long time = System.currentTimeMillis();
        Collections.sort(observations, observationComparator);
        logger.info("observations sorted in %d msec", System.currentTimeMillis() - time);
      }
  }
